;*******************************************************************************
;* TITLE	: JAGUAR.S						       *
;* Function	: System Initialization Routines			       *
;*									       *
;* Project #	: JAGUAR 3D pack out					       *
;*									       *
;* Programmer	: Cary J. Gee						       *
;*                                                                             *
;* COPYRIGHT 1992,1993 Atari U.S. Corporation				       *
;* UNATHORIZED REPRODUCTION, ADAPTATION, DISTRIBUTION, PERFORMANCE OR DISPLAY  *
;* OF THIS COMPUTER PROGRAM OR THE ASSOCIATED AUDIOVISUAL WORK IS STRICTLY     *
;* PROHIBITED.  ALL RIGHTS RESERVED.					       *
;*******************************************************************************

	.title	"JAGUAR.S - System Initialization Routines"

;*=============================================================================*
;* INCLUDE FILES							       *
;*=============================================================================*

BAR_ON EQU	0

	.include	'jaguar.inc'
	.include	'blit.inc'
	.include	'video.inc'
	.include	'objlist.inc'

;*=============================================================================*
;* EXTERNAL ROUTINES							       *
;*=============================================================================*

	.extern InitObjList
	.extern BuildObjList
	.extern CopyObjList

	.extern vidinit
	.extern vblank
	.extern vblinit

;*=============================================================================*
;* EXTERNAL DATA							       *
;*=============================================================================*

	.extern Objects

	.extern objData
	.extern lastObj
	.extern	dlistptr

	.extern olist
	.extern olist_ram

;*=============================================================================*
;* PUBLIC ROUTINES							       *
;*=============================================================================*

	.globl	InitJag

;*=============================================================================*
;* PUBLIC DATA								       *
;*=============================================================================*

;*=============================================================================*
;* LOCAL EQUATES							       *
;*=============================================================================*

;*******************************************************************************
;* SUBROUTINE AREA							       *
;*******************************************************************************

	.text

;*=============================================================================*
;* InitJag	- Initialize Jaguar video and object list		       *
;*									       *
;* Given:								       *
;*	Control								       *
;*									       *
;* Returns:								       *
;*	Initialises System						       *
;*									       *
;* Register Usage:							       *
;*	All Registers Preserved						       *
;*									       *
;*=============================================================================*

InitJag:
	movem.l	a0/d0,-(sp)		; save registers

	move.l	#$00070007,G_END	; set the endian thing for GPU
	move.l	#$00070007,D_END	; set the endian thing for DSP

	jsr	vidinit			; Initialize video
	jsr	vblinit			; Initialize VBlank routine

	move.l	#olist+15,d0		; force bi-phrase alignement
	andi.w	#~15,d0			; at first bi-phrase boundary in olist
	move.l	d0,dlistptr		; save for use by v_blank

	lea.l	Objects,a0		; ROM object list
	jsr	InitObjList		; Get the object list in RAM

	clr.l	d6			; build object list in RAM
	lea.l	objData,a1
	move.w	lastObj,d6
	add.w	#1,d6
	jsr 	BuildObjList

	jsr	CopyObjList		; initialise the object list for OLP

	move.l	dlistptr,d0		; use bi-phrase ptr to olist
	swap	d0			; and set the OLP
	move.l	d0,OLP

	move.w	#$6C1,VMODE		; Turn on display w/CRY ($4C7 for RGB)

	movem.l	(sp)+,a0/d0		; restore registers
	rts

;*******************************************************************************
;* END OF FILE								       *
;*******************************************************************************

	.end
